package se.lth.cs.dkinterface;

import se.lth.cs.dkinterface.control.MainController;
import se.lth.cs.dkinterface.model.DefaultsModel;
import se.lth.cs.dkinterface.model.DevicesModel;
import se.lth.cs.dkinterface.model.LibraryModel;
import se.lth.cs.dkinterface.model.PropertiesModel;
import se.lth.cs.dkinterface.model.ResultsListModel;
import se.lth.cs.dkinterface.model.StatusModel;
import se.lth.cs.dkinterface.services.connection.ConnectionServices;
import se.lth.cs.dkinterface.util.Dbg;
import se.lth.cs.dkinterface.view.DefaultsManagementView;
import se.lth.cs.dkinterface.view.DeviceDatabaseView;
import se.lth.cs.dkinterface.view.DeviceVerificationView;
import se.lth.cs.dkinterface.view.LibraryManagementView;
import se.lth.cs.dkinterface.view.StatusView;
import se.lth.cs.dkinterface.view.base.Screen;
import se.lth.cs.dkinterface.view.base.listview.ListView;

public class DKApplication
{
    public static void main(String[] args)
    {
        // Screen
        Screen screen = new Screen(new ListView());

        // Services
        ConnectionServices connectionServices = new ConnectionServices();
        connectionServices.start();

        // Models
        DevicesModel devicesModel       = new DevicesModel(connectionServices);    
        PropertiesModel propertiesModel = new PropertiesModel(connectionServices);
        LibraryModel libraryModel       = new LibraryModel(connectionServices);
        DefaultsModel defaultsModel     = new DefaultsModel(connectionServices);
        ResultsListModel resultsModel   = new ResultsListModel(connectionServices);
        StatusModel statusModel         = new StatusModel(connectionServices, libraryModel, devicesModel);

        // Views
        StatusView statusView                         = new StatusView(statusModel);
        DeviceDatabaseView deviceDatabaseView         = new DeviceDatabaseView(devicesModel, propertiesModel);
        DeviceVerificationView deviceVerificationView = new DeviceVerificationView(resultsModel, defaultsModel, libraryModel);
        DefaultsManagementView defaultsManagementView = new DefaultsManagementView(defaultsModel);
        LibraryManagementView libraryManagementView   = new LibraryManagementView(libraryModel);    

        // Listeners    
        connectionServices.addConnectionListener(devicesModel);
        connectionServices.addConnectionListener(propertiesModel);
        connectionServices.addConnectionListener(libraryModel);
        connectionServices.addConnectionListener(defaultsModel);
        connectionServices.addConnectionListener(resultsModel);
        connectionServices.addConnectionListener(statusModel);

        statusModel.addModelListener(statusView);

        Dbg.addLogListener(statusView);

        // Controller
        MainController controller = new MainController(screen);

        // Add views
        controller.addView(statusView);
        controller.addView(deviceDatabaseView);
        controller.addView(libraryManagementView);
        controller.addView(deviceVerificationView);
        controller.addView(defaultsManagementView);

        // Initialize and bring up interface
        controller.start();
    }
}
