package se.lth.cs.dkinterface.model;

import java.util.Vector;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;

import se.lth.cs.dkinterface.services.connection.ConnectionListener;
import se.lth.cs.dkinterface.services.connection.ConnectionServices;
import se.lth.cs.dkserver.model.database.DefaultsData;
import se.lth.cs.dkserver.services.server.command.Request;


public class DefaultsModel extends AbstractTableModel implements ConnectionListener
{
  private final String[] columnNames = {"Property", "Scope", "Value", "New value", "Reason"};
  private Object[][] defaults = { {"", "", "", "", "" } };

  private final Vector listeners = new Vector();

  private final ConnectionServices connectionServices;
  
  public DefaultsModel(ConnectionServices connectionServices)
  {
    this.connectionServices = connectionServices;
  }
  
  public void requestUpdate()
  {
    connectionServices.postCommand(Request.MESSAGE_TYPE_GET_DEFAULTS, "Get default data", "");
  }
  
  public void newDefault(String property, String scope, String value, String newValue, String reason)
  {
    String newDefault = property + ";" + scope + ";" + value + ";" + newValue + ";" + reason;
    connectionServices.postCommand(Request.MESSAGE_TYPE_NEW_DEFAULT, newDefault, newDefault);
  }
  
  public void deleteDefault(String property, String scope, String value, String newValue, String reason)
  {
    String delDefault = property + ";" + scope + ";" + value + ";" + newValue + ";" + reason;
    connectionServices.postCommand(Request.MESSAGE_TYPE_DEL_DEFAULT, delDefault, delDefault);
  }  
  
  @Override
  public void addTableModelListener(TableModelListener l) {
    listeners.add(l);
  }    

  @Override
  public void removeTableModelListener(TableModelListener l) {
    listeners.remove(l);
  }

  public int getColumnCount() {
    return columnNames.length;
  }

  public int getRowCount() {
    return defaults.length;
  }

  @Override
  public String getColumnName(int col) {
    return columnNames[col];
  }

  public Object getValueAt(int row, int col) {
    return defaults[row][col];
  }

  /*
   * JTable uses this method to determine the default renderer/
   * editor for each cell.  If we didn't implement this method,
   * then the last column would contain text ("true"/"false"),
   * rather than a check box.
   */
  @Override
  public Class getColumnClass(int c) {
    return getValueAt(0, c).getClass();
  }

  /*
   * Don't need to implement this method unless your table's
   * editable.
   */
  @Override
  public boolean isCellEditable(int row, int col) {
    //Note that the data/cell address is constant,
    //no matter where the cell appears onscreen.
    return true;
    /*  
      if (col < 2) {
            return false;
        } else {
            return true;
        }
     */
  }

  /*
   * Don't need to implement this method unless your table's
   * data can change.
   */
  @Override
  public void setValueAt(Object value, int row, int col) {
    defaults[row][col] = value;
    fireTableCellUpdated(row, col);
  }

  @Override
  public void newMessage(int type, Object message)
  {
    if(type == Request.MESSAGE_TYPE_DEFAULTS)
    {
      this.defaults = ((DefaultsData) message).defaults;

      for(int k = 0; k < listeners.size(); k++) {
        TableModelListener listener = (TableModelListener) listeners.elementAt(k);
        listener.tableChanged(new TableModelEvent(this)); //, ListDataEvent.CONTENTS_CHANGED, 0,  - 1));
      }      
    }
  } 

}
