package se.lth.cs.dkinterface.model;

import java.util.Vector;

import javax.swing.ListModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import se.lth.cs.dkinterface.services.connection.ConnectionListener;
import se.lth.cs.dkinterface.services.connection.ConnectionServices;
import se.lth.cs.dkserver.services.server.command.Request;

public class DevicesModel extends AbstractModel implements ConnectionListener, ListModel
{
  public static final String NAME = "DevicesModel";
  
  private final ConnectionServices connectionServices;
  private Vector<String> devices = new Vector<String>();
  private final Vector<ListDataListener> listeners = new Vector<ListDataListener>();
  
  public DevicesModel(ConnectionServices connectionServices)
  {
    setModelName(NAME);
    this.connectionServices = connectionServices;
  }
  
  public void requestUpdate()
  {
    connectionServices.postCommand(Request.MESSAGE_TYPE_GET_DEVICES, "Get device data", "");
  }
  
  public void requestPropertiesUpdate(int deviceIndex)
  {
    connectionServices.postCommand(Request.MESSAGE_TYPE_GET_PROPERTIES, "Get device propeties data", devices.elementAt(deviceIndex));
  }

  
  
  @Override
  public void newMessage(int type, Object message)
  {
    if(type == Request.MESSAGE_TYPE_DEVICES)
    {
      String[] data = ((String) message).split(";");
      Vector<String> devs = new Vector<String>();
      for(int k = 0; k < data.length; k++) {
        devs.add(data[k]);
      }
      this.devices = devs;
    
      // Tell all listeners that all content has to be reloaded.
      for(int k = 0; k < listeners.size(); k++) {
        ListDataListener listener = listeners.elementAt(k);
        listener.contentsChanged(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, getSize() - 1));
      }    
    }
  }

  public void addListDataListener(ListDataListener l) {
    listeners.add(l);
  }

  public Object getElementAt(int index) {
    return devices.elementAt(index);
  }

  public int getSize() {
    return devices.size();
  }

  public void removeListDataListener(ListDataListener l) {
    listeners.remove(l);
  }
}
