package se.lth.cs.dkinterface.model;

import java.util.Vector;

import javax.swing.ComboBoxModel;
import javax.swing.ListModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import se.lth.cs.dkinterface.services.connection.ConnectionListener;
import se.lth.cs.dkinterface.services.connection.ConnectionServices;
import se.lth.cs.dkinterface.util.Dbg;
import se.lth.cs.dkserver.services.server.command.Request;

public class LibraryModel extends AbstractModel implements ConnectionListener, ListModel 
{
  public static final String NAME = "LibraryModel";
  
  private final ConnectionServices connectionServices;
  
  private ParserModel parserModel;
  
  Vector<ListDataListener> listeners = new Vector<ListDataListener>();
  Vector<Object> libraries = new Vector<Object>();

  public LibraryModel(ConnectionServices connectionServices) {
    this.connectionServices = connectionServices;
    parserModel = new ParserModel();
  }      
  
  public ParserModel getParserModel()
  {
    return parserModel;
  }
  
  public void addNewLocation(String location, String parser)
  {
    connectionServices.postCommand(Request.MESSAGE_TYPE_NEW_LOC, "NEW_LOC", "[NEW_LOC]" + location + "[PARSER]" + parser + "[END]");
  }
  
  public void deleteLocation(String location, String parser)
  {
    connectionServices.postCommand(Request.MESSAGE_TYPE_DEL_LOC, "DEL_LOC", "[DEL_LOC]" + location + "[PARSER]" + parser + "[END]");
  }  
  
  public void requestLibraryUpdate()
  {
    connectionServices.postCommand(Request.MESSAGE_TYPE_GET_LOC, "GET_LOCATIONS", "GET_LOCATIONS");
  }
  
  public void requestLibraryReload()
  {
    connectionServices.postCommand(Request.MESSAGE_TYPE_RELOAD, "RELOAD_LIB_DATA", "RELOAD_LIB_DATA");
  }

  public void requestRestrictionsRebuild()
  {
    connectionServices.postCommand(Request.MESSAGE_TYPE_REBUILD, "REBUILD_RESTRICTIONS", "REBUILD_RESTRICTIONS");
  }  
  
  public void requestParsers()
  {
    connectionServices.postCommand(Request.MESSAGE_TYPE_GET_PARSERS, "GET_PARSERS", "GET_PARSERS");
  }    
  
  public void addListDataListener(ListDataListener l) {
    listeners.add(l);
  }

  public Object getElementAt(int index) {
    return libraries.elementAt(index);
  }

  public int getSize() {
    return libraries.size();
  }

  public void removeListDataListener(ListDataListener l) {
    listeners.remove(l);
  }

  @Override
  public void newMessage(int type, Object message)
  {
    if(type == Request.MESSAGE_TYPE_LOCS)
    {
      String newData = ((String) message).substring(6); // Remove 'ACKLOC'
      String[] data = newData.split(";");
      libraries = new Vector<Object>();
      for(int k = 0; k < data.length; k++) {
        libraries.add(data[k]);
        Dbg.println(data[k]);
      }
      
      // Tell all listeners that all content has to be reloaded.
      for(int k = 0; k < listeners.size(); k++) {
        ListDataListener listener = listeners.elementAt(k);
        listener.contentsChanged(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, getSize() - 1));
      }
    }
    else if(type == Request.MESSAGE_TYPE_PARSERS)
    {
      parserModel.setParsers(((String) message).split(";"));
    }
  }
  
  class ParserModel implements ComboBoxModel
  {
    Vector<ListDataListener> listeners = new Vector<ListDataListener>();
    String[] parsers = null;
    String selected = "";
    
    public void setParsers(String[] parsers)
    {
      this.parsers = parsers;
      if(getSize() > 0)
      {
        setSelectedItem(getElementAt(0));
      }
      fireChanged();
    }
    
    @Override
    public Object getSelectedItem()
    {
      return selected;
    }

    @Override
    public void setSelectedItem(Object arg0)
    {
      selected = (String) arg0;
      fireChanged();
    }

    @Override
    public void addListDataListener(ListDataListener l)
    {
      listeners.add(l);
    }

    @Override
    public Object getElementAt(int index)
    {
      return parsers[index];
    }

    @Override
    public int getSize()
    {
      return parsers == null ? 0 : parsers.length;
    }

    @Override
    public void removeListDataListener(ListDataListener l)
    {
      listeners.remove(l);
    }
    
    private void fireChanged()
    {
      for(int k = 0; k < listeners.size(); k++) {
        ListDataListener listener = listeners.elementAt(k);
        listener.contentsChanged(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, getSize() - 1));
      }        
    }
    
  }

}