package se.lth.cs.dkinterface.model;

import java.util.Vector;

import se.lth.cs.dkinterface.services.connection.ConnectionListener;
import se.lth.cs.dkinterface.services.connection.ConnectionServices;
import se.lth.cs.dkserver.services.server.command.Request;

public class StatusModel extends AbstractModel implements ConnectionListener
{
  private final ConnectionServices connectionServices;
  private final Vector<String> serverLogEntries;
  
  private final LibraryModel libraryModel;
  private final DevicesModel devicesModel;
  
  public StatusModel(ConnectionServices connectionServices,
                     LibraryModel libraryModel,
                     DevicesModel devicesModel)
  {
    this.connectionServices = connectionServices;
    this.devicesModel = devicesModel;
    this.libraryModel = libraryModel;
    serverLogEntries = new Vector<String>();
  }
  
  public int getNbrLibraries()
  {
    return libraryModel.getSize();
  }
  
  public int getNbrDevices()
  {
    return devicesModel.getSize();
  }
  
  public boolean connected()
  {
    return connectionServices.connected();
  }
  
  public synchronized Vector<String> getNewServerLogEntries()
  {
    Vector<String> tmp = new Vector<String>();
    for(String entry : serverLogEntries)
    {
      tmp.add(entry);
    }
    serverLogEntries.setSize(0);
    return tmp;
  }
  
  @Override
  public synchronized void newMessage(int type, Object message)
  {
    if(type == Request.MESSAGE_TYPE_LOG_ENTRY)
    {
      serverLogEntries.add((String) message);
      fireModelChanged();
    }
  }
}
