package se.lth.cs.dkinterface.services.connection;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.Vector;

import se.lth.cs.dkinterface.services.ConnectionReader;
import se.lth.cs.dkinterface.services.ConnectionWriter;
import se.lth.cs.dkinterface.util.Dbg;
import se.lth.cs.dkserver.services.server.command.Request;

public class ConnectionServices extends Thread 
{
  ObjectOutputStream out = null;
  ObjectInputStream in = null;  
  
  Vector<Request> inbox = new Vector<Request>();
  Vector<Request> outbox = new Vector<Request>();      
  
  Vector<ConnectionListener> listeners = new Vector<ConnectionListener>();
  
  private boolean connected;
  
  @Override
  public void run() 
  {
    try 
    {
      Dbg.println("UIClient.run(): Connecting to DKServer..");
      Socket dkSocket = new Socket("localhost", 4444);
      Dbg.println("UIClient.run(): Connected!");
      connected = true;
      
      out = new ObjectOutputStream(dkSocket.getOutputStream());
      in = new ObjectInputStream(dkSocket.getInputStream());            
      
      ConnectionReader reader = new ConnectionReader(in, inbox);
      new Thread(reader).start();
      ConnectionWriter writer = new ConnectionWriter(out, outbox);
      new Thread(writer).start();              
      
      
      while(true) 
      {
        if(inbox.size() > 0) 
        {
          Request incoming = inbox.remove(0);
          fireNewMessage(incoming.type, incoming.data);
        } 
        else 
        {
          try 
          {
            sleep(200);
          } 
          catch(Exception e) 
          { 
            Dbg.println(e.toString()); 
          }
        }
      }
    } 
    catch(Exception e) 
    { 
      Dbg.println(e.toString()); 
    }
  }

  public boolean connected()
  {
    return connected;
  }
  
  public void postCommand(int type, String info, Object data) {
    outbox.add(new Request(type, info, data));
  }
  
  public void addConnectionListener(ConnectionListener listener)
  {
    listeners.add(listener);
  }
  
  private void fireNewMessage(int type, Object message)
  {
    for(ConnectionListener listener : listeners)
    {
      listener.newMessage(type, message);
    }
  }
}
