package se.lth.cs.dkinterface.view;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.miginfocom.swing.MigLayout;
import se.lth.cs.dkinterface.model.DefaultsModel;
import se.lth.cs.dkinterface.view.base.AbstractView;
import se.lth.cs.dkinterface.view.components.ScrollingTable;

public class DefaultsManagementView extends AbstractView implements ActionListener 
{
  public static final String NAME = "Defaults management";
  
  private final JButton deleteButton;
  private final DefaultsModel defaultsModel;
  private final DefaultsScrollingTable defaultsScrollingTable;
  
  public DefaultsManagementView(DefaultsModel defaultsModel) 
  {
    setViewName(NAME);  
    this.defaultsModel = defaultsModel;
    
    MigLayout layout = new MigLayout("aligny top");
    setLayout(layout);

    JLabel mainHeader = new JLabel("Defaults management", JLabel.CENTER);
    mainHeader.setFont(new Font("", Font.BOLD, 20));
    mainHeader.setOpaque(true);
    mainHeader.setBorder(BorderFactory.createEtchedBorder());
    mainHeader.setBackground(new Color(0xCADCEB));
    add(mainHeader, "width 100%, height 50::, wrap");        
    
    defaultsScrollingTable = new DefaultsScrollingTable();
    add(defaultsScrollingTable, "width 100%, height 200:200:200, wrap");

    AddNewPanel newPanel = new AddNewPanel();    
    add(newPanel, "width 100%, wrap 30");
    
    deleteButton = new JButton("Delete selected rule");
    deleteButton.addActionListener(newPanel);
    add(deleteButton);
  }
  
  class DefaultsScrollingTable extends ScrollingTable implements ListSelectionListener
  {      
    public DefaultsScrollingTable() 
    {      
      super("Current default values", defaultsModel);
    }

    public void valueChanged(ListSelectionEvent e) { }
     
  }
  
  class AddNewPanel extends JPanel implements ActionListener 
  {
    JTextField propertyField = new JTextField();
    JTextField scopeField = new JTextField();
    JTextField valueField = new JTextField();
    JTextField newValueField = new JTextField();
    JTextField reasonField = new JTextField();
    
    JButton submitButton = new JButton("Add default rule");    
    
    public AddNewPanel() 
    {
      MigLayout migLayout = new MigLayout("aligny top");
      setLayout(migLayout);

      setBorder(BorderFactory.createTitledBorder("Create new rule"));        
      
      add(new JLabel("Property", JLabel.CENTER), "width 100::");
      add(new JLabel("Scope", JLabel.CENTER), "width 100::");
      add(new JLabel("Value", JLabel.CENTER), "width 100::");
      add(new JLabel("New value", JLabel.CENTER), "width 100::");
      add(new JLabel("Reason", JLabel.CENTER), "width 100::");
      
      submitButton = new JButton("Submit");
      submitButton.addActionListener(this);
      add(submitButton, "width 100::, span 1 2, gapleft 30, wrap");      
      
      propertyField = new JTextField("");
      add(propertyField, "width 100::");

      scopeField = new JTextField("");
      add(scopeField, "width 100::");

      valueField = new JTextField("");
      add(valueField, "width 100::");

      newValueField = new JTextField("");
      add(newValueField, "width 100::");

      reasonField = new JTextField("");
      add(reasonField, "width 100::");
    }  
    
    public JComboBox createReasonComboBox() 
    {
      return new JComboBox( new String[] { "Required" } );      
    }    
    
    public void actionPerformed(ActionEvent e) 
    {
      if(e.getSource() == submitButton) 
      {
        defaultsModel.newDefault(propertyField.getText(), scopeField.getText(), valueField.getText(), newValueField.getText(), reasonField.getText());
        propertyField.setText("");
        scopeField.setText("");
        valueField.setText("");
        newValueField.setText("");
        reasonField.setText("");
      } 
      else if(e.getSource() == deleteButton) 
      {
        JTable table = defaultsScrollingTable.getTable(); 
        int row = table.getSelectedRow();
        
          defaultsModel.deleteDefault("" + table.getValueAt(row, 0), 
                                      "" + table.getValueAt(row, 1), 
                                      "" + table.getValueAt(row, 2), 
                                      "" + table.getValueAt(row, 3), 
                                      "" + table.getValueAt(row, 4));

      } 
    }
  }

  @Override
  protected void activated()
  {
    defaultsModel.requestUpdate();
  }

  @Override
  protected void deactivated() { }

  @Override
  public void actionPerformed(ActionEvent e)
  {
    
  }

}
