package se.lth.cs.dkinterface.view;

import java.awt.Color;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.miginfocom.swing.MigLayout;
import se.lth.cs.dkinterface.model.AbstractModel;
import se.lth.cs.dkinterface.model.DevicesModel;
import se.lth.cs.dkinterface.model.ModelListener;
import se.lth.cs.dkinterface.model.PropertiesModel;
import se.lth.cs.dkinterface.view.base.AbstractView;
import se.lth.cs.dkinterface.view.components.DeviceListCellRenderer;
import se.lth.cs.dkinterface.view.components.ScrollingList;

public class DeviceDatabaseView extends AbstractView implements ModelListener 
{
  public static final String NAME = "Device database";    
  
  private final DevicesScrollingList devicesList;
  private final PropertiesScrollingList propertiesList;
  
  private final DevicesModel devicesModel;
  private final PropertiesModel propertiesModel;
  
  public DeviceDatabaseView(DevicesModel devicesModel, PropertiesModel propertiesModel) 
  {
    setViewName(NAME);

    this.devicesModel = devicesModel;
    this.propertiesModel = propertiesModel;
    
    MigLayout migLayout = new MigLayout();
    setLayout(migLayout);
    
    JLabel mainHeader = new JLabel("Device database", JLabel.CENTER);
    mainHeader.setFont(new Font("", Font.BOLD, 20));
    mainHeader.setOpaque(true);
    mainHeader.setBorder(BorderFactory.createEtchedBorder());
    mainHeader.setBackground(new Color(0xCADCEB));
    add(mainHeader, "width 100%, height 50::, span 2, wrap");    
    
    devicesList = new DevicesScrollingList();
    add(devicesList, "width 100%, height 100%");

    propertiesList = new PropertiesScrollingList();
    add(propertiesList, "width 100%, height 100%");    
  }
  
  class DevicesScrollingList extends ScrollingList implements ListSelectionListener 
  {
    public DevicesScrollingList() 
    {      
      super("Devices", devicesModel);
      getList().setCellRenderer(new DeviceListCellRenderer());
      getList().addListSelectionListener(this);
    }

    public void valueChanged(ListSelectionEvent e) 
    {
      devicesModel.requestPropertiesUpdate(devicesList.getList().getSelectedIndex());
    }  
  }

  class PropertiesScrollingList extends ScrollingList 
  {
    public PropertiesScrollingList() 
    {
      super("Properties", propertiesModel);
      getList().setCellRenderer(new DeviceListCellRenderer());      
    }   
  }

  @Override
  protected void activated() 
  { 
    devicesModel.requestUpdate();
  }

  @Override
  protected void deactivated() { }

  @Override
  public void modelChanged(AbstractModel model) {  }  

}