package se.lth.cs.dkinterface.view;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.miginfocom.swing.MigLayout;
import se.lth.cs.dkinterface.model.DefaultsModel;
import se.lth.cs.dkinterface.model.LibraryModel;
import se.lth.cs.dkinterface.model.ResultsListModel;
import se.lth.cs.dkinterface.view.base.AbstractView;
import se.lth.cs.dkinterface.view.components.ScrollingList;
import se.lth.cs.dkinterface.view.components.VerificationCellRenderer;
import se.lth.cs.dkserver.model.device.DeviceStatus;

public class DeviceVerificationView extends AbstractView 
{
  public static final String NAME = "Device verification";      

  private ScrollingList resultsList   = null;
  private final ResultsListModel resultsModel;
  private final DefaultsModel defaultsModel;
  private final LibraryModel libraryModel;

  public DeviceVerificationView(ResultsListModel resultsModel, DefaultsModel defaultsModel, LibraryModel libraryModel) 
  {
    setViewName(NAME);
    this.resultsModel = resultsModel;
    this.defaultsModel = defaultsModel;
    this.libraryModel = libraryModel;

    MigLayout layout = new MigLayout("aligny top");
    setLayout(layout);

    JLabel mainHeader = new JLabel("Device verification", JLabel.CENTER);
    mainHeader.setFont(new Font("", Font.BOLD, 20));
    mainHeader.setOpaque(true);
    mainHeader.setBorder(BorderFactory.createEtchedBorder());
    mainHeader.setBackground(new Color(0xCADCEB));
    add(mainHeader, "width 100%, height 50::, span 2, wrap");    
    
    ResultsPanel resultsPanel = new ResultsPanel();
    add(resultsPanel, "height 100%");

    ButtonPanel buttonPanel = new ButtonPanel();
    add(buttonPanel, "height 100%, width 200::, wrap");    
    
    UsagePanel usagePanel = new UsagePanel();
    add(usagePanel);    
  }

  class UsagePanel extends JPanel
  {
    public UsagePanel()
    {
      MigLayout layout = new MigLayout("aligny top");
      setLayout(layout);

      JTextArea textArea = new JTextArea();
      textArea.setEditable(false);
      textArea.setBackground(getBackground());
      textArea.setText("To enter new default values for a device which has failed verification, double-click it.");    
      add(textArea);
    }
  }
  
  class ButtonPanel extends JPanel implements ActionListener
  {
    private JButton reloadButton;
    private JButton rebuildButton;
    
    public ButtonPanel()
    {
      MigLayout layout = new MigLayout("alignx left, aligny top, insets 20");
      setLayout(layout);
      
      reloadButton = new JButton("Reload libraries");
      rebuildButton = new JButton("Rebuild constraints");
      reloadButton.addActionListener(this);
      rebuildButton.addActionListener(this);
      
      add(reloadButton, "width 150::, height 25::, wrap 15");
      add(rebuildButton, "width 150::, height 25::");
    }
    
    public void actionPerformed(ActionEvent e) 
    {
      if(e.getSource() == reloadButton) 
      {
        libraryModel.requestLibraryReload();  
      }
      else if(e.getSource() == rebuildButton) 
      {
        libraryModel.requestRestrictionsRebuild();
      }       
    }    
  }

  class ResultsPanel extends JPanel implements ListSelectionListener, ClickListener 
  {
    public ResultsPanel() 
    {
      MigLayout layout = new MigLayout("aligny top");
      setLayout(layout);

      resultsList = new ScrollingList("Verification status", resultsModel);
      resultsList.getList().setCellRenderer(new VerificationCellRenderer());
      resultsList.getList().addListSelectionListener(this);
      resultsList.addClickListener(this);
      add(resultsList, "width 500, height 100%");
    }

    public void valueChanged(ListSelectionEvent e) { }

    @Override
    public void indexClicked(Object object)
    {
      showAddDefaultsWindow((DeviceStatus) object);
    }  

    public void showAddDefaultsWindow(DeviceStatus deviceStatus)
    {
      if(deviceStatus.getMissingRequired().size() < 1)
      {
        return;
      }
      NewDefaultsWindow window = new NewDefaultsWindow(deviceStatus);
      window.setLocation(getLocationOnScreen().x + (getBounds().width - window.getWidth()) / 2, getLocationOnScreen().y + (getBounds().height - window.getHeight()) / 2);
    }

    class NewDefaultsWindow extends JFrame
    {
      
      
      public NewDefaultsWindow(DeviceStatus deviceStatus)
      {
        super("Enter default values for type " + deviceStatus.getDeviceType());
        JPanel panel = new JPanel();
        
        MigLayout migLayout = new MigLayout("aligny top");
        panel.setLayout(migLayout);

        JPanel headers = new JPanel(new MigLayout("aligny top, insets 0"));
        headers.add(new JLabel("Property", JLabel.CENTER), "width 100::");
        headers.add(new JLabel("Scope", JLabel.CENTER), "width 100::");
        headers.add(new JLabel("Value", JLabel.CENTER), "width 100::");
        headers.add(new JLabel("New value", JLabel.CENTER), "width 100::");
        headers.add(new JLabel("Reason", JLabel.CENTER), "width 100::");
        headers.add(new JLabel("", JLabel.CENTER), "width 100::");    
        panel.add(headers, "wrap");
        
        for(String missingRequired : deviceStatus.getMissingRequired())
        {
          panel.add(new DefaultEntry(missingRequired, deviceStatus.getDeviceType(), "<missing>", "", "Required"), "wrap");
        }

        setContentPane(panel);
        pack();      
        setVisible(true);
      }
      
      class DefaultEntry extends JPanel implements ActionListener
      {
        private final JTextField propertyField;
        private final JTextField scopeField;
        private final JTextField valueField;
        private final JTextField newValueField;
        private final JTextField reasonField;
        
        private final JButton submitButton;
        
        public DefaultEntry(String property, String scope, String value, String newValue, String reason)
        {
          MigLayout migLayout = new MigLayout("aligny top, insets 0 0 0 0");
          setLayout(migLayout);
          
          propertyField = new JTextField(property);
          propertyField.setEditable(false);
          add(propertyField, "width 100::");

          scopeField = new JTextField(scope);
          scopeField.setEditable(false);
          add(scopeField, "width 100::");

          valueField = new JTextField(value);
          valueField.setEditable(false);
          add(valueField, "width 100::");

          newValueField = new JTextField(newValue);
          add(newValueField, "width 100::");

          reasonField = new JTextField(reason);
          reasonField.setEditable(false);
          add(reasonField, "width 100::");

          submitButton = new JButton("Submit");
          submitButton.addActionListener(this);
          add(submitButton, "width 75::");          
        }

        @Override
        public void actionPerformed(ActionEvent e)
        {
          if(e.getSource() == submitButton && newValueField.getText().length() > 0)
          {
            defaultsModel.newDefault(propertyField.getText(), scopeField.getText(), valueField.getText(), newValueField.getText(), reasonField.getText());
            newValueField.setEditable(false);
            submitButton.setEnabled(false);
          }
        }   
      }
    }
  }

  @Override
  protected void activated()
  {
    resultsModel.requestRefreshResults();
  }

  @Override
  protected void deactivated() { }

}
