package se.lth.cs.dkinterface.view;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import net.miginfocom.swing.MigLayout;
import se.lth.cs.dkinterface.model.LibraryModel;
import se.lth.cs.dkinterface.view.base.AbstractView;
import se.lth.cs.dkinterface.view.components.LibraryList;
import se.lth.cs.dkinterface.view.components.ScrollingTextArea;

public class LibraryManagementView extends AbstractView 
{
  public static final String NAME = "Device libraries";      
  
  ScrollingTextArea logArea = null;
  LibraryModel libraryModel;
  
  public LibraryManagementView(LibraryModel libraryModel) 
  {
    setViewName(NAME);
    
    this.libraryModel = libraryModel;
    
    MigLayout layout = new MigLayout();
    setLayout(layout);

    JLabel mainHeader = new JLabel("Device libraries", JLabel.CENTER);
    mainHeader.setFont(new Font("", Font.BOLD, 20));
    mainHeader.setOpaque(true);
    mainHeader.setBorder(BorderFactory.createEtchedBorder());
    mainHeader.setBackground(new Color(0xCADCEB));
    add(mainHeader, "width 100%, height 50::, wrap");
    
    AddLocationPanel locPanel = new AddLocationPanel();
    LibraryList libList = new LibraryList("Currently registered libraries (devicedb : locations)", libraryModel, 600, 300);

    add(locPanel, "width 600, height 100, wrap");
    add(libList, "width 600, height 300");    
  }
  
  class AddLocationPanel extends JPanel implements ActionListener 
  {
    JTextField locTextField = new JTextField();
    JComboBox parserComboBox = createParserComboBox();
    JButton submitButton = new JButton("Add");    
    
    public AddLocationPanel() 
    {
      MigLayout layout = new MigLayout();
      setLayout(layout);
 
      setBorder(BorderFactory.createTitledBorder("Add new library"));  
      
      JLabel locLabel = new JLabel("Library location", SwingConstants.LEFT);
      JLabel parserLabel = new JLabel("Parser", SwingConstants.LEFT);

      add(locLabel, "width 200::, height 25::");
      add(parserLabel, "width 200::, height 25::, wrap");
      add(locTextField, "width 200::, height 25::");
      add(parserComboBox, "width 200::, height 25::");
      add(submitButton, "width 50::, height 25::");
      submitButton.addActionListener(this);
    }

    public JComboBox createParserComboBox() 
    {
      libraryModel.requestParsers();
      return new JComboBox( libraryModel.getParserModel() );
    }
    
    public void actionPerformed(ActionEvent e) 
    {
      if(e.getSource() == submitButton) 
      {
        String selected = (String) parserComboBox.getSelectedItem();
        libraryModel.addNewLocation(locTextField.getText(), selected.substring(0, selected.indexOf("Parser")));
      } 
    }
  }

  @Override
  protected void activated() 
  {
    libraryModel.requestLibraryUpdate();
  }

  @Override
  protected void deactivated() {  }

}
