package se.lth.cs.dkinterface.view;

import java.awt.Color;
import java.awt.Font;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.miginfocom.swing.MigLayout;
import se.lth.cs.dkinterface.model.AbstractModel;
import se.lth.cs.dkinterface.model.ModelListener;
import se.lth.cs.dkinterface.model.StatusModel;
import se.lth.cs.dkinterface.util.DebugListener;
import se.lth.cs.dkinterface.view.base.AbstractView;
import se.lth.cs.dkinterface.view.components.ScrollingTextArea;

public class StatusView extends AbstractView implements DebugListener, ModelListener
{
  public static final String NAME = "Status overview";
  
  private final StatusModel statusModel;
  private final StatusPanel statusPanel;
  private final ScrollingTextArea log;
  private final ScrollingTextArea server;
  
  public StatusView(StatusModel statusModel)
  {
    setViewName(NAME);
    
    this.statusModel = statusModel;
    
    MigLayout migLayout = new MigLayout();
    setLayout(migLayout);     

    JLabel mainHeader = new JLabel("DKServer Control Interface", JLabel.CENTER);
    mainHeader.setFont(new Font("", Font.BOLD, 20));
    mainHeader.setOpaque(true);
    mainHeader.setBorder(BorderFactory.createEtchedBorder());
    mainHeader.setBackground(new Color(0xA1DFAF));
    add(mainHeader, "width 100%, height 50::, span 2, wrap");

    statusPanel = new StatusPanel(new String[][] { { "Connection:" , "Not connected" },  
                                                   { "Server:" , "localhost" },
                                                   { "Port:" , "4444" },
                                                   { "Libraries:" , "-" },
                                                   { "Devices:" , "-" }
                                                 }); 
    add(statusPanel, "height 50%");
    
    log = new ScrollingTextArea("Log");
    add(log, "width 100%, height 50%, wrap");

    add(new ConnectPanel(), "height 50%");
    
    server = new ScrollingTextArea("Server");
    add(server, "width 100%, height 50%");    
  }
  
  class StatusPanel extends JPanel 
  {
    Hashtable<String, JLabel> labels = new Hashtable<String, JLabel>();
    
    public StatusPanel(String[][] data) 
    {
      setBorder(BorderFactory.createTitledBorder("Status"));    

      MigLayout migLayout = new MigLayout("aligny top");
      setLayout(migLayout);      
      
      for(int k = 0; k < data.length; k++)
      {
        add(new JLabel(" " + data[k][0]), "width 100::");
        JLabel label = new JLabel(data[k][1]);
        add(label, "width 150::, wrap");
        labels.put(data[k][0], label);
      }
    }
    
    public void setPropertyValue(String property, String value)
    {
      labels.get(property).setText(value);
      repaint();
    }
    
  }

  class ConnectPanel extends JPanel 
  {
    private final JTextField serverField;
    private final JTextField portField;
    private final JButton connectButton;
    
    public ConnectPanel() 
    {
      setBorder(BorderFactory.createTitledBorder("Connection"));    

      MigLayout migLayout = new MigLayout("aligny top");
      setLayout(migLayout);      

      add(new JLabel("Server:", JLabel.LEFT), "width 100::");
      
      serverField = new JTextField("localhost");
      add(serverField, "width 150::, wrap");
      
      add(new JLabel("Port:", JLabel.LEFT), "width 100::");
      
      portField = new JTextField("4444");
      add(portField, "width 150::, wrap 10");
      
      connectButton = new JButton("Connect!");
      add(connectButton, "width 100%, span 2");
      
      serverField.setEnabled(false);
      portField.setEnabled(false);
      connectButton.setEnabled(false);
      
    }
  }  
  
  @Override
  public void activated() 
  {
    statusPanel.setPropertyValue("Connection:", statusModel.connected() ? "Connected" : "Not connected");
    statusPanel.setPropertyValue("Libraries:", String.valueOf(statusModel.getNbrLibraries()));
    statusPanel.setPropertyValue("Devices:", String.valueOf(statusModel.getNbrDevices()));
  }

  @Override
  public void deactivated() { }

  @Override
  public void newLogEntry(String s)
  {
    log.append(s);
  }

  @Override
  public void modelChanged(AbstractModel model)
  {
    Vector<String> logEntries = statusModel.getNewServerLogEntries();
    for(String entry : logEntries)
    {
      server.append(entry);
    }
  }
  
}
