package se.lth.cs.dkinterface.view.base.listview;

import java.util.Vector;

import javax.swing.JSplitPane;
import javax.swing.ListModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import se.lth.cs.dkinterface.view.base.AbstractView;

public class ListView extends JSplitPane implements ListSelectionListener
{
  private ListViewList displayList;
  private ListViewPanel displayFrame;

  private ListContents listContents;
  private Vector<AbstractView> views;

  public ListView()
  {
    super(JSplitPane.HORIZONTAL_SPLIT);
    
    views = new Vector<AbstractView>();
    listContents = new ListContents();
    
    displayList = new ListViewList(listContents);
    displayList.addListSelectionListener(this);
    displayFrame = new ListViewPanel();
    
    setLeftComponent(displayList);
    setRightComponent(displayFrame);
    
    setResizeWeight(1);
    setDividerLocation((int) displayList.getMinimumSize().getWidth());
    this.setResizeWeight(0);
    setOneTouchExpandable(true);
    setContinuousLayout(true);    
  }

  public void addView(AbstractView view)
  {
    views.add(view);
    listContents.addDisplay(view);
    displayFrame.addView(view);
  }  

  @Override
  public void valueChanged(ListSelectionEvent event)
  {
    String sel = (String) displayList.getSelectedValue();
    displayFrame.viewActivated(views.elementAt(displayList.getSelectedIndex()));
  }  
  
  class ListContents implements ListModel
  {
    Vector<ListDataListener> listeners = new Vector<ListDataListener>();
   
    public void addDisplay(AbstractView view) {
      for(ListDataListener listener : listeners)
      {
        listener.contentsChanged(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, getSize() - 1));
      }
    }    
    
    @Override
    public void addListDataListener(ListDataListener l)
    {
      listeners.add(l);
    }

    @Override
    public Object getElementAt(int index)
    {
      return views.elementAt(index).getViewName();
    }

    @Override
    public int getSize()
    {
      return views.size();
    }

    @Override
    public void removeListDataListener(ListDataListener l)
    {
      listeners.remove(l);
    }  
  }


}
