package se.lth.cs.dkserver;

import se.lth.cs.dkserver.model.SettingsModel;
import se.lth.cs.dkserver.model.database.DatabaseModel;
import se.lth.cs.dkserver.model.ontology.OntologyModel;
import se.lth.cs.dkserver.model.ontology.ProtegeOntologyModel;
import se.lth.cs.dkserver.services.loader.LibraryServices;
import se.lth.cs.dkserver.services.server.ControlServer;
import se.lth.cs.dkserver.services.verification.FallbackServices;
import se.lth.cs.dkserver.services.verification.VerificationServices;
import se.lth.cs.dkserver.util.Dbg;

public class DKServer 
{
    // Start server
    // - Run startup sequence for library loader (possibly check and/or reload libraries)
    // - Start control server accepting external commands
    public static void main(String[] args) 
    {
        SettingsModel settingsModel = new SettingsModel("settings.xml");

        DatabaseModel databaseModel = new DatabaseModel(settingsModel);

        OntologyModel ontologyModel = new ProtegeOntologyModel(settingsModel);

        FallbackServices fallbackServices = new FallbackServices(databaseModel);

        VerificationServices verificationServices = new VerificationServices(fallbackServices, ontologyModel);

        LibraryServices loader = new LibraryServices(databaseModel, ontologyModel, verificationServices);
        loader.reload();

        ControlServer controlServer = new ControlServer(loader, verificationServices, databaseModel);
        Dbg.addLogListener(controlServer);

        controlServer.start();
    }

}
