package se.lth.cs.dkserver.model;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import se.lth.cs.dkserver.util.Dbg;

public class SettingsModel
{
    private final String SETTINGS_FILE; 
    private Properties settings;
    
    public SettingsModel(String file)
    {
        SETTINGS_FILE = file;
        load();
    }
    
    public void writeSetting(String setting, String value)
    {
        Dbg.println("Writing: " + setting + ", " + value);
        settings.put(setting, value);
        store();
    }
    
    public String readSetting(String setting)
    {
        Dbg.println("Reading: " + setting + ", " + settings.getProperty(setting));
        return settings.getProperty(setting);        
    }    
    
    public void load()
    {
        settings = new Properties();
        try
        {
            settings.loadFromXML(new FileInputStream(SETTINGS_FILE));
        }
        catch(FileNotFoundException fnfe)
        {
            Dbg.println("SettingsModel.load(): File not found: " + SETTINGS_FILE + ", creating..");
            store();
        }
        catch(IOException ioe)
        {
            Dbg.println("SettingsModel.load(): IOException: " + SETTINGS_FILE);
        }
    }
    
    public void store()
    {
        try
        {
            settings.storeToXML(new FileOutputStream(SETTINGS_FILE), "Settings");
        }
        catch(IOException ioe)
        {
            Dbg.println("SettingsModel.store(): IOException: " + SETTINGS_FILE);
        }        
    }
}
