package se.lth.cs.dkserver.model.database;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import se.lth.cs.dkserver.model.SettingsModel;
import se.lth.cs.dkserver.model.device.Device;
import se.lth.cs.dkserver.model.device.Property;
import se.lth.cs.dkserver.settings.SettingsManager;
import se.lth.cs.dkserver.util.Dbg;

public class DatabaseModel 
{
    private final String DB_NAME;
    private final String DB_USER;
    private final String DB_PASS;
    private final String DB_HOST;
    
    public DatabaseModel(SettingsModel settingsModel) 
    {        
        DB_HOST = settingsModel.readSetting("dbhost");
        DB_NAME = settingsModel.readSetting("dbname");
        DB_USER = settingsModel.readSetting("dbuser");
        DB_PASS = settingsModel.readSetting("dbpass");
        initDriver();
        if(!tablesCreated())
        {
            Dbg.println("Incomplete table set, creating..");
            runBatchFile("tables.sql");
        }
    }  

    private boolean tablesCreated()
    {
        //Connection con = getConnection("localhost", "devicedb", "dkserver", "julmust");
        Connection con = getConnection();
        String[][] res = executeQuery(con, "SELECT TABLE_NAME FROM information_schema.tables WHERE table_schema = '" + DB_NAME + "'", new String[] { "TABLE_NAME" });
        try 
        {
            con.close();
        } 
        catch(SQLException e) 
        { 
            e.printStackTrace();
        }

        for(int k = 0; k < res.length; k++)
        {
            Dbg.println("Found table: " + res[k][0]);
        }
        
        if(res.length < 4) 
        {
            return false;
        } 
        return true;
    }
    
    public static void main(String[] args) 
    {
        DatabaseModel m = new DatabaseModel(new SettingsModel("settings.xml"));
        System.out.println(m.tablesCreated());
        //m.runBatchFile("tables.sql");
    }

    /*
     * Get new value for an invalid property if possible, otherwise return null.
     */
    public String getDefaultValue(String propertyType, String scope, String value) 
    {
        Connection con = getConnection();
        String[][] res = executeQuery(con, "SELECT * FROM defaults WHERE property = '" + propertyType + "' AND scope = '" + scope + "' AND value = '" + value + "'", new String[] { "newValue" });
        try 
        {
            con.close();
        } 
        catch(SQLException e) 
        { 
            Dbg.println(e.toString()); 
        }      
        if(res.length > 0) 
        {
            return res[0][0];
        } 
        else 
        {
            return null;
        }
    }


    public String[][] getProperties(String owner) 
    {
        Dbg.print("MySQLManager.getProperties(): Fetching..");
        Connection con = getConnection();
        String[][] res = executeQuery(con, "SELECT * FROM properties WHERE owner = '" + owner + "'", new String[] { "type", "value" });
        try 
        {
            con.close();
        } 
        catch(SQLException e) 
        { 
            Dbg.println(e.toString()); 
        }  
        Dbg.println("Done (" + res.length + ")");
        return res;
    }

    public String getPropertiesAsString(String owner) 
    {
        String[][] props = getProperties(owner);
        String str = "";
        for(int k = 0; k < props.length-1; k++) 
        {
            str = str + props[k][0] + " | " + props[k][1] + ";";
        }
        str = str + props[props.length-1][0] + " | " + props[props.length-1][1];
        return str;
    }  


    public String[][] getDefaults() 
    {
        Dbg.print("MySQLManager.getDefaults(): Fetching..");
        Connection con = getConnection();
        String[][] res = executeQuery(con, "SELECT * FROM defaults", new String[] { "property", "scope", "value", "newValue", "reason" });
        try 
        {
            con.close();
        } 
        catch(SQLException e) 
        { 
            Dbg.println(e.toString()); 
        }  
        Dbg.println("Done (" + res.length + ")");
        return res;
    }

    public String[][] getDevices() 
    {
        Dbg.println("MySQLManager.getDevices(): Fetching..");
        Connection con = getConnection();
        String[][] res = executeQuery(con, "SELECT * FROM devices", new String[] { "name" });
        try 
        {
            con.close();
        } 
        catch(SQLException e) 
        { 
            Dbg.println(e.toString()); 
        }  
        return res;
    }

    public String getDevicesAsString() 
    {
        String[][] devs = getDevices();
        String str = "";
        if(devs.length > 0)
        {
            for(int k = 0; k < devs.length-1; k++) 
            {
                str = str + devs[k][0] + ";";
            }
            str = str + devs[devs.length-1][0];
        }
        return str;
    }  


    /*
     * Get all registered device library locations and associated parsers.
     */
    public String[][] getLocationsAndParsers() 
    {
        Dbg.print("MySQLManager.getLocationsAndParsers(): Fetching..");
        Connection con = getConnection();
        String[][] res = executeQuery(con, "SELECT * FROM locations", new String[] { "location", "parser" });
        try 
        {
            con.close();
        } 
        catch(SQLException e) 
        { 
            Dbg.println(e.toString()); 
        }  
        Dbg.println("Done (" + res.length + ")");
        return res;
    }  

    public String getLocationsAndParsersAsString() 
    {
        String[][] locs = getLocationsAndParsers();
        String str = "";
        for(int k = 0; k < locs.length-1; k++) 
        {
            str = str + locs[k][0] + " | " + locs[k][1] + ";";
        }
        str = str + locs[locs.length-1][0] + " | " + locs[locs.length-1][1];
        return str;
    }

    public void addNewDefault(String[] data) 
    {
        Dbg.println("MySQLManager.addNewDefault(): Inserting...");
        Connection con = getConnection();
        try 
        {
            // TODO: Add checks for duplicate entries.
            Statement s = con.createStatement();
            // Insert location and parser into locations table.
            s.execute("INSERT INTO defaults (property,scope,value,newValue,reason) VALUES ('" + data[0] + "','" 
                    + data[1] + "','"
                    + data[2] + "','"
                    + data[3] + "','"
                    + data[4] + "')");
            con.close();
        } 
        catch(Exception e) 
        { 
            Dbg.println(e.toString()); 
        }
    }  

    public void deleteDefault(String[] data) 
    {
        Dbg.println("MySQLManager.deleteDefault(): Deleting...");
        Connection con = getConnection();
        try 
        {
            // TODO: Add checks for duplicate entries.
            Statement s = con.createStatement();
            // Insert location and parser into locations table.
            s.execute("DELETE FROM defaults WHERE property='" + data[0] + "' AND scope='" + data[1] + "' AND value='" + data[2] + "' AND newValue='" + data[3] + "' AND reason='" + data[4] + "'");
            con.close();
        } 
        catch(Exception e) 
        { 
            Dbg.println(e.toString()); 
        }
    }    

    public void addLocationAndParser(String location, String parser) 
    {
        Dbg.println("MySQLManager.addLocationAndParser(): Inserting: " + location + ", " + parser);
        Connection con = getConnection();
        try 
        {
            // TODO: Add checks for duplicate entries.
            Statement s = con.createStatement();
            // Insert location and parser into locations table.
            s.execute("INSERT INTO locations (location,parser) VALUES ('" + location + "','" 
                    + parser + "')");
            con.close();
        } 
        catch(Exception e) 
        { 
            Dbg.println(e.toString()); 
        }
    }

    public void delLocationAndParser(String location, String parser) 
    {
        Dbg.println("MySQLManager.delLocationAndParser(): Deleting: " + location + ", " + parser);
        Connection con = getConnection();
        try 
        {
            Statement s = con.createStatement();
            s.execute("DELETE FROM locations WHERE location='" + location + "' AND parser='" + parser + "'");
            con.close();
        } 
        catch(Exception e) 
        { 
            Dbg.println(e.toString()); 
        }
    }  

    public void clearLoc(String location) 
    {
        Dbg.println("MySQLManager.clearLoc(): Clearing location: " + location);
        Connection con = getConnection();    
        try 
        {
            Statement s = con.createStatement();

            s.execute("DELETE devices, properties FROM devices INNER JOIN properties ON devices.name = properties.owner WHERE devices.location = '" + location +"'");

            //s.execute("DELETE FROM devices, properties WHERE devices.location = '" + location + "' AND devices.name = properties.owner");
            con.close();
        } 
        catch(Exception e) 
        { 
            Dbg.println(e.toString()); 
        }  
    }

    public void insertDevice(Device d) 
    {
        Dbg.println("MySQLManager.insertDevice(): Inserting device: " + d.toString());
        Connection con = getConnection();    
        try 
        {
            // TODO: Add checks for duplicate entries.
            Statement s = con.createStatement();
            // Insert device name, type and location into devices table.
            s.execute("INSERT INTO devices (name,type,location) VALUES ('" + d.getName() + "','" 
                    + d.getType() + "','" 
                    + d.getLocation() + "')");
            for(Property p : d.getProperties())
            {
                // Insert property data into properties table.
                s.execute("INSERT INTO properties (owner,type,value) VALUES ('" + p.owner + "','" 
                        + p.type + "','"  
                        + p.value + "')");

            }
            con.close();
        } 
        catch(Exception e) 
        { 
            Dbg.println(e.toString()); 
        }
    }

    /*
     * Run batch file of sql statements.
     */
    public void runBatchFile(String file) 
    {
        Dbg.println("MySQLManager.runBatchFile(): " + file);    
        Connection con = getConnection();
        try 
        {
            Statement s = con.createStatement();
            Vector<String> v = SettingsManager.readSettingsFile(file);
            for(String sqlStr : v) 
            {
                s.execute(sqlStr);
            }
            con.close();
        } 
        catch(Exception e) 
        { 
            Dbg.println(e.toString()); 
        } 
    }

    /*
     * Query database.
     */
    private String[][] executeQuery(Connection con, String sqlStatement, String[] columnHeaders) {
        Vector<String[]> v = new Vector<String[]>();
        String[][] res;
        try {
            Statement s = con.createStatement();
            ResultSet rs = s.executeQuery(sqlStatement);
            while(rs.next()) {
                String[] row = new String[columnHeaders.length];
                for(int k = 0; k < columnHeaders.length; k++) {
                    row[k] = (rs.getObject(columnHeaders[k]).toString());
                }
                v.add(row);
            }
            rs.close();
            res = new String[v.size()][columnHeaders.length];
            for(int k = 0; k < res.length; k++)
                res[k] = v.elementAt(k);
        } catch(SQLException e) {
            Dbg.println("MySQLManager.executeQuery(): Error executing SQL statement.");
            Dbg.println(e.toString());
            return null;
        }
        return res;
    }  

    /*
     *  Initialize driver.
     */
    private void initDriver() 
    {
        try 
        {
            Class.forName("com.mysql.jdbc.Driver").newInstance();
            Dbg.println("MySQLManager.initDriver(): MySQL Driver Found.");
        } 
        catch(Exception e) 
        {
            Dbg.println("MySQLManager.initDriver(): MySQL JDBC Driver not found.");
            Dbg.println(e.toString());
        }
    }  

    private Connection getConnection()
    {
        return getConnection(DB_HOST, DB_NAME, DB_USER, DB_PASS);
    }
    
    private Connection getConnection(String host, String db, String user, String pass) 
    {
        try 
        {
            return DriverManager.getConnection("jdbc:mysql://" + host + "/" + db + "?user=" + user + "&password=" + pass);
        } 
        catch(java.sql.SQLException e) 
        {
            Dbg.println("MySQLManager.getConnection(): Connection couldn't be established.");
            Dbg.println(e.toString());
            return null;
        }
    }  

}