package se.lth.cs.dkserver.model.device;

import java.util.Vector;

public class Device {

  private String name;
  private String type;
  private Vector<Property> properties;
  private String location;
  private String status = "";
  private DeviceStatus deviceStatus;
  
  public Device(String name, String type, String location) {
    this.name = name;
    this.type = type;
    properties = new Vector<Property>();
    this.location = location;
    deviceStatus = new DeviceStatus(name, type);
  }
  
  public Device() {
    this.name = null;
    this.type = null;
    properties = new Vector<Property>();
    this.location = null;
    deviceStatus = new DeviceStatus(name, type);
  }  
  
  public void setName(String name)
  {
    this.name = name;
    deviceStatus.setDeviceName(name);
  }

  public void setType(String type)
  {
    this.type = type;
    deviceStatus.setDeviceType(type);
  }

  public void setProperties(Vector<Property> properties)
  {
    this.properties = properties;
  }

  public void setLocation(String location)
  {
    this.location = location;
  }

  public void setStatus(String status)
  {
    this.status = status;
  }

  public void setDeviceStatus(DeviceStatus deviceStatus)
  {
    this.deviceStatus = deviceStatus;
  }

  public String getName()
  {
    return name;
  }

  public String getType()
  {
    return type;
  }

  public Vector<Property> getProperties()
  {
    return properties;
  }

  public String getLocation()
  {
    return location;
  }

  public String getStatus()
  {
    return status;
  }

  public DeviceStatus getDeviceStatus()
  {
    return deviceStatus;
  }

  public void addProperty(Property property) {
    properties.add(property);
  }

  public boolean sufficient() {
    boolean propertiesSufficient = true;
    for(Property p : properties) {
      if(!p.sufficient()) {
        propertiesSufficient = false;
        break;
      }
    }
    return name != null && type != null && propertiesSufficient && location != null;
  }
  
  public boolean complete() {
    return sufficient() && properties.size() > 0;
  }   
 
  @Override
  public String toString() {
    String str = "";
    str += "[" + name + " (";
    for(Property p : properties) {
      str += p.type + " ";
    }
    str = str.trim();
    str += ")]";
    return str;
  }
  
}
