package se.lth.cs.dkserver.services.parser;

import java.io.File;
import java.util.Vector;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.methods.GetMethod;

import se.lth.cs.dkserver.model.device.Device;
import se.lth.cs.dkserver.model.device.DeviceLoadException;
import se.lth.cs.dkserver.model.device.Property;
import se.lth.cs.dkserver.settings.SettingsManager;
import se.lth.cs.dkserver.util.Dbg;

public class PlainTextFormat1Parser implements DeviceLibraryParser 
{
  public Device[] getDevices(String location) throws DeviceLoadException 
  {
    Dbg.println("  PlainTextFormatParser1Parser.getDevice(): Loading " + location);    
    String data = null;
    if(location.startsWith("http://")) 
    {  // TODO: Better check.
      Dbg.println("  PlainTextFormatParser1Parser.getDevice(): Location is: webserver");
      // We have a file on a webserver.
      HttpClient client = new HttpClient();
      HttpMethod method = new GetMethod(location);
      byte[] responseBody = null;
      try 
      {
        client.executeMethod(method);
        responseBody = method.getResponseBody();
      } 
      catch(Exception e) 
      { 
        Dbg.println(e.toString()); 
      }
      method.releaseConnection();
      data = new String(responseBody);
    } 
    else 
    {  // TODO: More alternatives.
      Dbg.println("  PlainTextFormatParser1Parser.getDevice(): Location is: local file");
      // Local file.
      File txtFile = new File(location);
      if(!txtFile.exists()) 
      {
        throw new DeviceLoadException("Local file '" + location + "' does not exist.");
      }
      data = SettingsManager.readFileToString(txtFile);
    }
    // Here we have loaded the device library into the String variable 'data'.
    // Do some cleaning up, removing white space etc.
    String[] lines = data.split("\n");
    for(int k = 0; k < lines.length; k++) 
    {
      lines[k] = lines[k].trim();
      lines[k] = lines[k].replaceAll("\n","");
      //Dbg.println(lines[k]);
    }
    // Parse the text and create Device objects to return.
    
    Vector<Vector<String[]>> devices = new Vector<Vector<String[]>>();
    for(int k = 0; k < lines.length; k++) 
    {
      lines[k] = lines[k].replace('|', ';');
      String[] entry = lines[k].split(";");
      for(int i = 0; i < entry.length; i++) 
      {
        entry[i] = entry[i].trim();
      }
      
      boolean added = false;
      for(Vector<String[]> device : devices) 
      {
        // If name of first entry of device equals entry being added,
        // we have found correct device to add to 
        if(device.elementAt(0)[0].equalsIgnoreCase(entry[0])) 
        {
          device.add(entry);
          added = true;
          break;
        }
      }
      if(!added) 
      {
        Vector<String[]> newDevice = new Vector<String[]>();
        newDevice.add(entry);
        devices.add(newDevice);
      }
    }
    Vector<Device> dbDevices = new Vector<Device>();
    for(Vector<String[]> device : devices) 
    {
      Device dbDevice = new Device();
      dbDevice.setLocation(location);
      for(String[] entry : device) 
      {
        dbDevice.setName(entry[0]); // Unnecessary to do every time, but easy way of doing it.
        if(entry[1].equalsIgnoreCase("Class")) 
        {
          dbDevice.setType(entry[2]);
        } 
        else 
        {
          dbDevice.addProperty(new Property(dbDevice.getName(), entry[1], entry[2]));
        }
      }
      if(dbDevice.sufficient()) 
      {
        dbDevices.add(dbDevice);
        //Dbg.println("Added device: [" + dbDevice.name + ":" + dbDevice.type + "] (" + dbDevice.properties.size() + " properties)");
      } 
      else 
      {
        throw new DeviceLoadException("Device and/or properties could not be properly read.");
      }
    }
    Dbg.println("PlainTextFormatParser1Parser.getDevice(): Done loading " + dbDevices.size() + " device(s)");
    return dbDevices.toArray(new Device[dbDevices.size()]);    
    
    
    /*
    Device device = new Device();
    device.location = location;
    for(int k = 0; k < lines.length; k++) {
      String str = lines[k];
      if(str.startsWith("Device")) {
        Dbg.println("  PlainTextFormatParser1Parser.getDevice(): Found device");        
        // Device class
        device.type = str.substring(str.indexOf(":")+1, str.indexOf(",")).trim();
        // Device name
        device.name = str.substring(str.indexOf(",")+1).trim();
      } else if(str.startsWith("Property")) {
        Dbg.println("  PlainTextFormatParser1Parser.getDevice(): Found property");
        Property property = new Property();
        property.type = str.substring(str.indexOf(":")+1, str.indexOf(",")).trim();
        str = str.substring(str.indexOf(",")+1);
        property.name = str.substring(0, str.indexOf(",")).trim();
        str = str.substring(str.indexOf(",")+1);
        property.datatype = str.substring(0, str.indexOf(",")).trim();
        str = str.substring(str.indexOf(",")+1);
        property.datavalue = str.trim();
        property.location = location;
        device.addProperty(property);
      }
    } */

  }

}
