package se.lth.cs.dkserver.services.server;

import java.util.Vector;

import se.lth.cs.dkserver.model.database.DatabaseModel;
import se.lth.cs.dkserver.model.database.DefaultsData;
import se.lth.cs.dkserver.services.loader.LibraryServices;
import se.lth.cs.dkserver.services.server.command.Request;
import se.lth.cs.dkserver.services.verification.VerificationServices;

public class MessageHandler
{
  private final Vector<Request> inbox;
  private final Vector<Request> outbox;
  
  private final LibraryServices loader;
  private final VerificationServices verificationServices;
  private final DatabaseModel databaseModel;
  
  public MessageHandler(Vector<Request> inbox, 
                        Vector<Request> outbox, 
                        LibraryServices loader,
                        VerificationServices verificationServices,
                        DatabaseModel databaseModel)
  {
    this.inbox = inbox;
    this.outbox = outbox;
    this.loader = loader;
    this.verificationServices = verificationServices;
    this.databaseModel = databaseModel;
  }
  
  public void handle()
  {
    Request incoming = inbox.remove(0);
    
    /*
     * Request to add new device library location to database
     */
    if(incoming.type == Request.MESSAGE_TYPE_NEW_LOC) 
    {
      String message = (String) incoming.data;
      String newLoc = message.substring(message.indexOf("]")+1, message.indexOf("[PARSER]"));
      String newParser = message.substring(message.indexOf("[PARSER]")+("[PARSER]".length()), message.indexOf("[END]"));
      databaseModel.addLocationAndParser(newLoc, newParser);
      outbox.add(new Request(Request.MESSAGE_TYPE_LOCS, 
          "ACKLOC" + databaseModel.getLocationsAndParsersAsString(), 
          "ACKLOC" + databaseModel.getLocationsAndParsersAsString()));      
    } 
    
    /*
     * Request to delete device library location from database
     */
    if(incoming.type == Request.MESSAGE_TYPE_DEL_LOC) 
    {
      String message = (String) incoming.data;
      String delLoc = message.substring(message.indexOf("]")+1, message.indexOf("[PARSER]"));
      String delParser = message.substring(message.indexOf("[PARSER]")+("[PARSER]".length()), message.indexOf("[END]"));
      databaseModel.delLocationAndParser(delLoc, delParser);
      outbox.add(new Request(Request.MESSAGE_TYPE_LOCS, 
          "ACKLOC" + databaseModel.getLocationsAndParsersAsString(), 
          "ACKLOC" + databaseModel.getLocationsAndParsersAsString()));      
      
    }    
    
    /*
     * Request for current library locations and their parsers
     */
    else if(incoming.type == Request.MESSAGE_TYPE_GET_LOC) 
    {
      outbox.add(new Request(Request.MESSAGE_TYPE_LOCS, 
                                   "ACKLOC" + databaseModel.getLocationsAndParsersAsString(), 
                                   "ACKLOC" + databaseModel.getLocationsAndParsersAsString()));
    }
    
    /*
     * Request that contents of libraries be reloaded
     * Flushes database and loads all information from library sources again
     */
    else if(incoming.type == Request.MESSAGE_TYPE_RELOAD) 
    {
      loader.reload();
    }
    
    /*
     * Request to get a list of all current devices
     */
    else if(incoming.type == Request.MESSAGE_TYPE_GET_DEVICES) 
    {
      String s1 = databaseModel.getDevicesAsString();
      Request req = new Request(Request.MESSAGE_TYPE_DEVICES, s1, s1);
      outbox.add(req);
    }
    
    /*
     * Request to get a list of all properties for a given device
     */
    else if(incoming.type == Request.MESSAGE_TYPE_GET_PROPERTIES) 
    {
      outbox.add(new Request(Request.MESSAGE_TYPE_PROPERTIES, 
                                   databaseModel.getPropertiesAsString((String) incoming.data), 
                                   databaseModel.getPropertiesAsString((String) incoming.data)));
    }
    
    /*
     * Request to get information ablout how the last verification went
     * I.e. information about the verification of each device
     */
    else if(incoming.type == Request.MESSAGE_TYPE_GET_RESULTS) 
    {
//      outbox.add(new Request(Request.MESSAGE_TYPE_RESULTS, 
//                                   verificationServices.getLatestResultsAsString(), 
//                                   verificationServices.getLatestResultsAsString()));
      outbox.add(new Request(Request.MESSAGE_TYPE_RESULTS, 
          "Verification results", 
          verificationServices.getDeviceResults()));      
    }
    
    /*
     * Request to rebuild restriction data from ontology
     */
    else if(incoming.type == Request.MESSAGE_TYPE_REBUILD) 
    {
      verificationServices.rebuildRestrictions(true);
      //loader.reload();
//      outbox.add(new Request(Request.MESSAGE_TYPE_RESULTS, 
//          "Verification results", 
//          verificationServices.getDeviceResults()));      
    }    
    
    /*
     * Request to get all defaults data, i.e. the data that should be used when trying
     * to avoid a device being discarded during verification
     */
    else if(incoming.type == Request.MESSAGE_TYPE_GET_DEFAULTS) 
    {
      outbox.add(new Request(Request.MESSAGE_TYPE_DEFAULTS, 
                                   "Defaults data", 
                                   new DefaultsData(databaseModel.getDefaults())));
    }
    
    /*
     * Request to get available parsers
     */
    else if(incoming.type == Request.MESSAGE_TYPE_GET_PARSERS) 
    {
      String[] parsers = loader.getParsers();//SettingsManager.getParserNames();
      String res = parsers[0];
      for(int k = 1; k < parsers.length; k++)
      {
        res += ";";
        res += parsers[k];
      }
      outbox.add(new Request(Request.MESSAGE_TYPE_PARSERS, 
                                   "Parsers", 
                                   res));
    }    
    
    /*
     * Set new user/pass for connecting to database
     */
    else if(incoming.type == Request.MESSAGE_TYPE_SET_DB_USERPASS) 
    {
        // Not implemented.
    }
    
    /*
     * Request to add new default data
     */
    else if(incoming.type == Request.MESSAGE_TYPE_NEW_DEFAULT) 
    {
      // Add new default rule
      String str = (String) incoming.data;
      databaseModel.addNewDefault(str.split(";"));
      outbox.add(new Request(Request.MESSAGE_TYPE_DEFAULTS, 
                                   "Defaults updated", 
                                   new DefaultsData(databaseModel.getDefaults())));
    }
    
    /*
     * Request to delete default data
     */
    else if(incoming.type == Request.MESSAGE_TYPE_DEL_DEFAULT) 
    {
      // Delete default rule
      String str = (String) incoming.data;
      databaseModel.deleteDefault(str.split(";"));
      outbox.add(new Request(Request.MESSAGE_TYPE_DEFAULTS, 
                                   "Defaults updated", 
                                   new DefaultsData(databaseModel.getDefaults())));
    }    
    
    
    else if(incoming.type == Request.MESSAGE_TYPE_LOG_ENTRY) 
    {
      outbox.add(incoming);
    }    
  }
}
