package se.lth.cs.dkserver.services.verification;

import se.lth.cs.dkserver.model.database.DatabaseModel;
import se.lth.cs.dkserver.model.device.Device;
import se.lth.cs.dkserver.model.device.Property;
import se.lth.cs.dkserver.util.Dbg;

public class FallbackServices 
{ 
  private final DatabaseModel databaseModel;
  
  public FallbackServices(DatabaseModel databaseModel)
  {
    this.databaseModel = databaseModel;
  }
  
  public Device handle(int errorType, Device d, String propertyType, String value) {
    switch(errorType) {
    case VerificationServices.MISSING_REQUIRED_PROPERTY: // some values from
      // Lookup default if available, otherwise discard device.
      String newValue = databaseModel.getDefaultValue(propertyType, d.getType(), "<missing>");
      if(newValue != null) {
        Property newProperty = new Property(d.getName(), propertyType, newValue);
        d.addProperty(newProperty);
        Dbg.println("      FallbackHandler.handle(): Got new value for missing required property: " + newValue + " (" + propertyType + "," + d.getType() + ")");
        return d;
      } else {
        // Not possible to get valid value for this property, discard device.
        Dbg.println("      FallbackHandler.handle(): Failed to get new value for missing required property. Device will be discarded.");
        return null;
      }
    case VerificationServices.PROPERTY_CLASS_MISMATCH:   // only values from (device spec has en entry which is not a Property)
      // Ignore property (delete). Handled by VerificationManager.
      break;
    case VerificationServices.PROPERTY_VALUE_MISMATCH:   // cast failure ((double) "heavy")
      // Lookup alternate value if available, otherwise 1) discard property (not required) or 2) discard device (property is required).
      
      break;
    case VerificationServices.FORBIDDEN_PROPERTY:        // isPropertyOf only
      // Discard property or device (user setting?)
      
      break;      
    }
    return null;
  }
  
}
