package se.lth.cs.dkserver.settings;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

import se.lth.cs.dkserver.services.parser.ParserFilter;
import se.lth.cs.dkserver.util.Dbg;
import edu.stanford.smi.protegex.owl.ProtegeOWL;
import edu.stanford.smi.protegex.owl.jena.JenaOWLModel;

public class SettingsManager
{  
    private static final String RESTRICTIONS_FOLDER = "restrictions";
    private static final String ONTOLOGY_FOLDER     = "ontology";
    private static final String PARSER_FOLDER       = "parsers";
    private static final String SETTINGS_FOLDER     = "settings";

    public static void createRestrictionFile(String contents, String name) 
    {
        writeStringToFile(contents, RESTRICTIONS_FOLDER + File.separator + name);
    }

    public static Vector<String> getRestrictionFile(String name) 
    {
        File resFile = new File(RESTRICTIONS_FOLDER + File.separator + name);
        if(resFile.exists()) 
        {
            return readFile(resFile);
        }
        return null;
    }

    public static String[] getParserNames()
    {
        String[] parsers = getAvailableParsers();
        for(int k = 0; k < parsers.length; k++)
        {
            parsers[k] = parsers[k].substring(0, parsers[k].indexOf("."));
        }
        return parsers;
    }

    public static String[] getAvailableParsers() 
    {
        File parserFolder = new File(PARSER_FOLDER);
        Dbg.println("SettingsManager.getAvailableParsers() Searching folder: '" + parserFolder.getAbsolutePath() + "'");
        return parserFolder.list(new ParserFilter());
    }

    public static String[] getDeviceRestrictionFiles() 
    {
        File restrictionsFolder = new File(RESTRICTIONS_FOLDER);
        return restrictionsFolder.list(new DeviceRestrictionFilter());
    }  

    public static JenaOWLModel loadOntologyFromFile(String file) 
    {
        JenaOWLModel ontology = null;
        try {
            String s = new File(ONTOLOGY_FOLDER + File.separator + file).toURI().toString();
            ontology = ProtegeOWL.createJenaOWLModelFromURI(s);
        } 
        catch(Exception e) 
        {
            Dbg.println("SettingsManager.loadOntologyFromFile(): Could not load OWL model.");
            ontology = null;
        }
        return ontology;
    }     

    public static Vector<String> readSettingsFile(String file) 
    {
        final char skipToken = ';';
        BufferedReader theReader = null;
        Vector<String> v = new Vector<String>();      
        try 
        {
            theReader = new BufferedReader(new FileReader(new File(SETTINGS_FOLDER + File.separator + file)));
            String line = theReader.readLine();
            while(line != null) 
            {
                if(line.length() > 0 && line.charAt(0) != skipToken)
                {
                    v.add(line);
                }
                line = theReader.readLine();       
            }
        } 
        catch(Exception e) 
        { 
            e.printStackTrace();
        }
        return v;
    }

    public static Vector<String> readFile(File fileURL) 
    {
        final char skipToken = ';';
        BufferedReader theReader = null;
        Vector<String> v = new Vector<String>();      
        try 
        {
            theReader = new BufferedReader(new FileReader(fileURL));
            String line = theReader.readLine();
            while(line != null) 
            {
                if(line.length() > 0 && line.charAt(0) != skipToken)
                    v.add(line);
                line = theReader.readLine();       
            }
        } 
        catch(Exception e) 
        { 
            e.printStackTrace(); 
        }
        return v;
    }  

    public static String readFileToString(File f) 
    {
        Vector<String> v = readFile(f);
        String s = "";
        for(String str : v) 
        {
            s += (str + "\n");
        }
        return s;
    }

    public static void writeTextToFile(Vector<String> rows, String filename) 
    {
        PrintWriter pw;
        try 
        {
            pw = new PrintWriter(new FileOutputStream(new File(filename)));
            for(int k = 0; k < rows.size(); k++) {
                pw.println(rows.elementAt(k));
            }
            pw.flush();
            pw.close();
        } 
        catch(IOException ioe) 
        {
            ioe.printStackTrace();
        }
    }  

    public static File writeStringToFile(String rows, String filename) 
    {
        PrintWriter pw;
        File txtFile = new File(filename);
        try 
        {
            pw = new PrintWriter(new FileOutputStream(txtFile));
            pw.println(rows);
            pw.flush();
            pw.close();
        } 
        catch(IOException ioe) 
        {
            ioe.printStackTrace();
        }
        return txtFile;
    }
}
